#Noesis python importer - EA NHL games .rx2 models - version 1.3
from inc_noesis import *
import noesis

#rapi methods should only be used during handler callbacks
import rapi
#registerNoesisTypes is called by Noesis to allow the script to register formats.
#Do not implement this function in script files unless you want them to be dedicated format modules!
 
def registerNoesisTypes():
	handle = noesis.register("NHL Legacy (X360) models v1.4", ".rx2")
	noesis.setHandlerTypeCheck(handle, noepyCheckType)
	noesis.setHandlerLoadModel(handle, noepyLoadModel)
	#noesis.setHandlerLoadRGBA(handle, noetexLoadRGBA)
	
	noesis.logPopup()
	#print("The log can be useful for catching debug prints from preview loads.\nBut don't leave it on when you release your script, or it will probably annoy people.")
	return 1
#check if it's this type based on the data
def noepyCheckType(data):
    bs = NoeBitStream(data)
    Magic = bs.readBytes(7)
    if Magic != b'\x89\x52\x57\x34\x78\x62\x32':
        return 0
    return 1
 
texList = []
def noetexLoadRGBA(data, texList):
   del texList[:]
   bs = NoeBitStream(data, NOE_BIGENDIAN) 
   baseName = rapi.getExtensionlessName(rapi.getLocalFileName(rapi.getLastCheckedName()))
   TexInfo = []
   TexOff = []
   TexBufferSize = []
   NameInfo = []
   NameOff = []
   TexNames = []
   bs.seek(0x10, NOESEEK_ABS)
   version = bs.readBytes(3)
   bs.seek(13, NOESEEK_REL)   
   fileCount = bs.read(">i")
   bs.seek(12, NOESEEK_REL)
   fileTable = bs.read(">i")
   bs.seek(16, NOESEEK_REL)
   hdrSize = bs.read(">i")
   bs.seek(fileTable[0], NOESEEK_ABS)
   texCount = 0
   names = 0
   #check file types and file info		   			   		
   for i in range(0, fileCount[0]):
   		fileInfo = bs.read(">iiiii")
   		fileType = bs.readBytes(4)
   		if fileType == b'\x00\x02\x00\x03':
	   		bs.seek(-48, NOESEEK_REL)
	   		info = bs.read(">iiiiiiiiiiii")
	   		TexOff.append([info[0]])
	   		TexBufferSize.append([info[2]])
	   		TexInfo.append([info[6] + 33])
	   		texCount +=1
	   			   	
   #Read all textures in texList		   			   		
   for i in range(0, texCount):
   		bs.seek(TexInfo[i][0], NOESEEK_ABS) #Seek to texture info
   		TexData = bs.read(">HBBBHiii") #Read WORD, BYTE, WORD, WORD, DWORD, DWORD, DWORD in Big endian
   		imgFmt = TexData[1]
   		imgHeight = (TexData[3] + 1) * 8
   		imgWidth = (TexData[4] + 1) & 0x1FFF

   		
   		TexNames.append(str(i + 1))
   		
   		bs.seek(TexOff[i][0] + hdrSize[0], NOESEEK_ABS) #Seek to file start address
   		data = bs.readBytes(TexBufferSize[i][0]) #Read texture buffer
   		texFmt = 0
   		#DXT1
   		if imgFmt == 0x52:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 8)
   			texFmt = noesis.NOESISTEX_DXT1
   		#DXT3
   		elif imgFmt == 0x53:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 16)
   			texFmt = noesis.NOESISTEX_DXT3
   		#DXT5
   		elif imgFmt == 0x54:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 16)
   			texFmt = noesis.NOESISTEX_DXT5
   		#DXT5 packed normal map
   		elif imgFmt == 0x71:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 16)
   			data = rapi.imageDecodeDXT(data, imgWidth, imgHeight, noesis.FOURCC_ATI2)
   			texFmt = noesis.NOESISTEX_RGBA32
   		#DXT1 packed normal map
   		elif imgFmt == 0x7C:
   			data = rapi.imageUntile360DXT(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 8)
   			data = rapi.imageDecodeDXT(data, imgWidth, imgHeight, noesis.FOURCC_DXT1NORMAL)
   			texFmt = noesis.NOESISTEX_RGBA32	
   		#raw a8r8g8b8
   		elif imgFmt == 0x86:
   			if "name" in baseName:
   				data = rapi.imageDecodeRaw(data, imgWidth, imgHeight, "a8r8g8b8")
   				texFmt = noesis.NOESISTEX_RGBA32
   			else:
   				data = rapi.imageUntile360Raw(data, imgWidth, imgHeight, 4)
   				data = rapi.imageDecodeRaw(data, imgWidth, imgHeight, "a8r8g8b8")
   				texFmt = noesis.NOESISTEX_RGBA32
   		#raw b5g6r5
   		elif imgFmt == 0x44:
   			data = rapi.imageUntile360Raw(rapi.swapEndianArray(data, 2), imgWidth, imgHeight, 2)
   			data = rapi.imageDecodeRaw(data, imgWidth, imgHeight, "b5g6r5")
   			texFmt = noesis.NOESISTEX_RGBA32

   		#unknown, not handled
   		else:
   			print("WARNING: Unhandled image format " + repr(imgFmt) + " - " + repr(imgWidth) + "x" + repr(imgHeight) + " - " + repr(len(data)))
   			return None
   				   			    			  	   					 		
   		tex1 = NoeTexture(TexNames[i], imgWidth, imgHeight, data, texFmt)
   		texList.append(tex1)
   		

   return 1
#load the model

def noepyLoadModel(data, mdlList):
   VInfo = []
   VOff = []
   VBufferSize = []
   FInfo = []   
   FOff = []
   FBufferSize = []
   MatInfo = []
   MatIndexInfo = []
   MatIndexTable = []
   TexIndexInfo = []
   TexIndex = []
   AmbientTex = []
   AmbientTexIndex = []
   MeshInfo = []
   DescID = []
   UVpos = []
   UVtype = []
   UV2pos = []
   UV2type = []
   COLpos = []
   COLtype = []
   VBsize = []
   VertType = []
   MatNames = []
   MatNameIndex = []
   NameInfo = []
   MatList = []
   MaterialInfo = []
   MeshNames = []
   
   ctx = rapi.rpgCreateContext()
   bs = NoeBitStream(data, NOE_BIGENDIAN)
   bs.seek(0x20, NOESEEK_ABS)
   fileCount = bs.read(">i")
   bs.seek(12, NOESEEK_REL)
   fileTable = bs.read(">i")
   bs.seek(16, NOESEEK_REL)
   hdrSize = bs.read(">i")  
   bs.seek(fileTable[0], NOESEEK_ABS)
   meshCount = 0
   texCount = 0
   materialCount = 0
   ebCount = 0
   descriptorCount = 0

   
   for i in range(0, fileCount[0]):
	    fileInfo = bs.read(">iiiii")
	    fileType = bs.readBytes(4)
	   			    
	    if fileType == b'\x00\x02\x00\x05':
	   		bs.seek(-48, NOESEEK_REL)
	   		info = bs.read(">iiiiiiiiiiii")
	   		VOff.append([info[0]])
	   		VBufferSize.append([info[2]])
	   		VInfo.append([info[6]])
	   		
	    elif fileType == b'\x00\x02\x00\x07':
	   		bs.seek(-48, NOESEEK_REL)
	   		info = bs.read(">iiiiiiiiiiii")
	   		FOff.append([info[0]])
	   		FBufferSize.append([info[2]])
	   		FInfo.append([info[6]])	   		
	    elif fileType == b'\x00\x02\x00\x03':
	   		texCount +=1		
	    elif fileType == b'\x00\x02\x00\x09':
	   		meshCount +=1 

	    elif fileType == b'\x00\x02\x00\x04':
	   		MeshInfo.append([fileInfo[0]])
	   		descriptorCount +=1
	   		DescID.append([i])
	   			   		
	    elif fileType == b'\x00\xEB\x00\x03':
	   		MatIndexInfo.append([fileInfo[0]])
	   		ebCount +=1
	   		
	    elif fileType == b'\x00\xEC\x00\x10':
	   		NameInfo.append([fileInfo[0]])
	    
	    elif fileType == b'\x00\xEF\x00\x01':
	   		TexIndexInfo.append([fileInfo[0]])
	   		if fileInfo[2] == 0x140:
	   			AmbientTex.append(1)
	   		else:
	   			AmbientTex.append(0)

	    elif fileType == b'\x00\xEF\x00\x05':
	   		MatInfo.append([fileInfo[0]])
	   		materialCount +=1				 			   		
   
#Read material names
   if ebCount != 0:
   		bs.seek(NameInfo[0][0], NOESEEK_ABS)
   		nameCount = bs.read(">HH")
   		bs.seek(4, NOESEEK_REL)
   		for i in range(0, nameCount[1]):
   			nameOff = bs.read(">i")
   			nameType = bs.readBytes(4)
   			if nameType == b'\x00\xEF\x00\x05':
   				MatNameIndex.append(bs.read(">i"))
   				pos = bs.tell()
   				bs.seek(nameOff[0], NOESEEK_ABS)
   				MatNames.append(bs.readString())
   				bs.seek(pos, NOESEEK_ABS)	
#Read material indices for meshes
   if materialCount != 0:
   		for i in range(0, ebCount):
   			bs.seek(MatIndexInfo[i][0] + 0x10, NOESEEK_ABS)
   			contents = bs.read(">i")
   			tablesize = bs.read(">i")
   			bs.seek(tablesize[0] + 12, NOESEEK_REL)
   			for j in range(0, contents[0]):
   				materialIndex = bs.read(">i")
   				MatIndexTable.append(materialIndex[0])
   				bs.seek(40, NOESEEK_REL)

#Read texture indices for materials
   if texCount != 0:
   		for i in range(0, materialCount):
   			bs.seek(TexIndexInfo[i][0] + 256, NOESEEK_ABS)
   			diffTex = bs.read(">i")
   			TexIndex.append(str(diffTex[0] // 2) + ".png")
   			if AmbientTex[i] == 1:
   				bs.seek(28, NOESEEK_REL)
   				ambTex = bs.read(">i")
   				AmbientTexIndex.append(str(ambTex[0] // 2) + ".png")
   			else:
   				AmbientTexIndex.append(str(0))
   				
   if texCount != 0:		
   		for i in range(0, materialCount):
   			print 
   			material = NoeMaterial(MatNames[i],"")
   			if texCount != 0:
   				material.setTexture(TexIndex[i])
   				if AmbientTex[i] == 1:
   					material.setOcclTexture(AmbientTexIndex[i])
   			MatList.append(material) 
		
   UVCount = 0
   UV2Count = 0
   COLCount = 0

   for i in range(0, meshCount):
   		if descriptorCount < meshCount:
   			bs.seek(VInfo[i][0] + 40, NOESEEK_ABS)
   			partID = bs.read(">i")
   			bs.seek(MeshInfo[partID[0] - DescID[0][0]][0], NOESEEK_ABS)
   		else:
   			bs.seek(MeshInfo[i][0], NOESEEK_ABS)
   		info = bs.read(">iiBBBBBBBB")
   		VBsize.append(info[3])
   		for j in range(0, info[2]):
   			position = bs.read(">i")
   			type = bs.readBytes(8)
   			if type == b'\x00\x2A\x23\xB9\x00\x00\x00\x01':
   				VertType.append(1)   			
   			elif type == b'\x00\x1A\x23\x60\x00\x00\x00\x01':
   				VertType.append(2)  			
   			elif type == b'\x00\x2C\x23\xA5\x00\x05\x00\x06':
   				UVpos.append(position[0])
   				UVtype.append(1)
   				UVCount += 1
   			elif type == b'\x00\x2C\x23\xA5\x00\x05\x01\x07':
   				UV2pos.append(position[0])
   				UV2type.append(1)
   				UV2Count += 1
   				
   			elif type == b'\x00\x2C\x23\x5F\x00\x05\x00\x06':
   				UVpos.append(position[0])
   				UVtype.append(2)
   				UVCount += 1
   			
   			elif type == b'\x00\x2C\x20\x59\x00\x05\x00\x06':
   				UVpos.append(position[0])
   				UVtype.append(3)
   				UVCount += 1   			

   			elif type == b'\x00\x2C\x21\x59\x00\x05\x00\x06':
   				UVpos.append(position[0])
   				UVtype.append(4)
   				UVCount += 1
   				
   			elif type == b'\x00\x18\x28\x86\x00\x0A\x00\x04':
   				COLpos.append(position[0])
   				COLtype.append(1)
   				COLCount += 1
   				 			
   		if UVCount == i:
   			UVtype.append(0)
   			UVpos.append(8)
   			UVCount += 1
   		if UV2Count == i:
   			UV2type.append(0)
   			UV2pos.append(8)
   			UV2Count += 1
   		if COLCount == i:
   			COLtype.append(0)
   			COLpos.append(8)
   			COLCount += 1	
   					  				
 #Construct Models		  			   
   for i in range(0, meshCount):
   		print(COLtype[i])
   		B = 0
   		bs.seek(VInfo[i][0] + 32, NOESEEK_ABS) #Seek to vertices info
   		VData = bs.read(">HHi")
   		VBsize = VData[0]
   		VCount = VData[2]
   		bs.seek(FOff[i][0] + hdrSize[0], NOESEEK_ABS)
   		TriStrip = 0
   		#check if the faces are triangleStrip format
   		checkBuffer = FBufferSize[i][0] // 2
   		for j in range(0, checkBuffer):
   			StripCheck = bs.read(">H")
   			if StripCheck[0] == 0xFFFF:
   				TriStrip = 1
   		bs.seek(FInfo[i][0], NOESEEK_ABS) #Seek to faces info
   		bs.seek(32, NOESEEK_REL)
   		FCount = bs.read(">HH")
   		#sometimes triangle count isn't divisible by 3. Subtract B from triangle count
   		if FCount [1] % 3 != 0:
   			B += 2  
   		bs.seek(VOff[i][0] + hdrSize[0], NOESEEK_ABS) #Seek to Vertices Start
   		VertBuff = bs.readBytes(VBufferSize[i][0])
   		rapi.rpgSetName("mesh_" + str(i))
   		rapi.rpgSetOption(noesis.RPGOPT_BIGENDIAN, 1)
   		if VertType[i] == 1:
   			rapi.rpgBindPositionBufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, 0)
   		if VertType[i] == 2:
   			rapi.rpgBindPositionBufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, 0)   		
   		if UVtype[i] == 1:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, UVpos[i])
   		if UVtype[i] == 2:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_HALFFLOAT, VBsize, UVpos[i])
   		if UVtype[i] == 3:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_USHORT, VBsize, UVpos[i])   			 			
   		if UVtype[i] == 4:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_SHORT, VBsize, UVpos[i])
   		if UVtype[i] == 0:
   			rapi.rpgBindUV1BufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, 8)	
   		if UV2type[i] == 1:
   			rapi.rpgBindUV2BufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, UV2pos[i])
   		if UV2type[i] == 0:
   			rapi.rpgBindUV2BufferOfs(VertBuff, noesis.RPGEODATA_FLOAT, VBsize, 8)	
   		if COLtype[i] == 1:
   			rapi.rpgBindColorBufferOfs(VertBuff, noesis.RPGEODATA_BYTE, VBsize, COLpos[i], 4)
   		
   		bs.seek(FOff[i][0] + hdrSize[0], NOESEEK_ABS) #Seek to Faces Start
   		FaceBuff = bs.readBytes(FBufferSize[i][0])
   		
   		if texCount != 0:
   			rapi.rpgSetMaterial(MatNames[(MatIndexTable[i] - MatNameIndex[0][0]) // 3])
   		if TriStrip == 1:
   			rapi.rpgCommitTriangles(FaceBuff, noesis.RPGEODATA_USHORT, (FCount[1]), noesis.RPGEO_TRIANGLE_STRIP, 1)
   		else:
   			rapi.rpgCommitTriangles(FaceBuff, noesis.RPGEODATA_USHORT, FCount[1] - B, noesis.RPGEO_TRIANGLE, 1)
   if texCount != 0:
   		noetexLoadRGBA(data, texList)
   if meshCount != 0:		
   		mdl = rapi.rpgConstructModel()
   if materialCount != 0:
   		mdl.setModelMaterials(NoeModelMaterials(texList, MatList))
   mdlList.append(mdl)
   rapi.rpgClearBufferBinds()
   rapi.rpgReset() 
   return 1
   